package it.polito.ferrovie;

import it.polito.ferrovie.bean.Connessione;
import it.polito.ferrovie.bean.Fermata;
import it.polito.ferrovie.bean.Linea;
import it.polito.ferrovie.dao.FerrovieDAO;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.jgrapht.alg.BellmanFordShortestPath;
import org.jgrapht.graph.DefaultDirectedWeightedGraph;
import org.jgrapht.graph.DefaultWeightedEdge;

import com.javadocmd.simplelatlng.LatLngTool;
import com.javadocmd.simplelatlng.util.LengthUnit;

public class Model {
	
	private List<Linea> linee ;
	private List<Fermata> fermate ;
	private List<Connessione> connessioni ;
	
	private FerrovieDAO dao = new FerrovieDAO();
	private DefaultDirectedWeightedGraph<Fermata, DefaultWeightedEdge> grafo = null;
	
	public void buildGraph() {
		fermate = dao.getAllFermata();
		linee = dao.getAllLinea();
		connessioni = dao.getAllConnessione(linee, fermate);
		
		grafo = new DefaultDirectedWeightedGraph<Fermata, DefaultWeightedEdge>(DefaultWeightedEdge.class);
		
		for (Fermata f: fermate) {
			grafo.addVertex(f);
		}
		
		for (Connessione c: connessioni) {
			DefaultWeightedEdge e = grafo.addEdge(c.getStazP(), c.getStazA());
			
			if (e!=null) {
				double velocita = c.getLinea().getVelocita();
				
				double distanza = LatLngTool.distance(c.getStazP().getCoords(), c.getStazA().getCoords(), LengthUnit.KILOMETER);
				
				double tempo = (distanza / velocita) * 60;
				
				grafo.setEdgeWeight(e, tempo);
			}
		}
		
		System.out.println("Grafo creato: " + grafo.vertexSet().size() + " nodi, " + grafo.edgeSet().size() + " archi");
		
	}
	
	public List<Linea> getLinee() {
		return linee;
	}
	
	public void setLinee(List<Linea> linee) {
		this.linee = linee;
	}
	
	public List<Fermata> getFermate() {
		return fermate;
	}
	
	public void setFermate(List<Fermata> fermate) {
		this.fermate = fermate;
	}
	
	public List<Connessione> getConnessioni() {
		return connessioni;
	}
	
	public void setConnessioni(List<Connessione> connessioni) {
		this.connessioni = connessioni;
	}

	public String calcolaPercorso(Fermata partenza) {
		String ritorno = new String();
		
		BellmanFordShortestPath<Fermata, DefaultWeightedEdge> bellman = 
				new BellmanFordShortestPath<Fermata, DefaultWeightedEdge>(grafo, partenza);
		
		Map<Fermata, Double> mappa = new HashMap();
		
		for (Fermata arrivo: grafo.vertexSet()) {
			if (!partenza.equals(arrivo)) {
				double peso = bellman.getCost(arrivo);
				mappa.put(arrivo, peso);
			}
		}
		
		for (Fermata f: mappa.keySet()) {
			ritorno = ritorno + f.getNome() + " = " + mappa.get(f) + "\n";
		}
		
		return ritorno;
	}
}
